@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Button.cxx,v 1.4.2.6.2.21 2003/01/30 21:41:30 easysw Exp $"
+// "$Id: Fl_Button.cxx 7903 2010-11-28 21:06:39Z matt $"
 //
 // Button widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2003 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 #include <FL/Fl.H>
@@ -28,26 +30,41 @@
 #include <FL/Fl_Group.H>
 #include <FL/Fl_Window.H>
 
+
+Fl_Widget_Tracker *Fl_Button::key_release_tracker = 0;
+
+
 // There are a lot of subclasses, named Fl_*_Button.  Some of
 // them are implemented by setting the type() value and testing it
 // here.  This includes Fl_Radio_Button and Fl_Toggle_Button
 
+/**
+  Sets the current value of the button.
+  A non-zero value sets the button to 1 (ON), and zero sets it to 0 (OFF).
+  \param[in] v button value.
+  \see set(), clear()
+ */
 int Fl_Button::value(int v) {
   v = v ? 1 : 0;
   oldval = v;
   clear_changed();
   if (value_ != v) {
     value_ = v;
-    redraw();
+    if (box()) redraw();
+    else redraw_label();
     return 1;
   } else {
     return 0;
   }
 }
 
+/**
+  Turns on this button and turns off all other radio buttons in the group
+  (calling \c value(1) or \c set() does not do this).
+ */
 void Fl_Button::setonly() { // set this radio button on, turn others off
   value(1);
-  Fl_Group* g = (Fl_Group*)parent();
+  Fl_Group* g = parent();
   Fl_Widget*const* a = g->array();
   for (int i = g->children(); i--;) {
     Fl_Widget* o = *a++;
@@ -58,16 +75,21 @@ void Fl_Button::setonly() { // set this radio button on, turn others off
 void Fl_Button::draw() {
   if (type() == FL_HIDDEN_BUTTON) return;
   Fl_Color col = value() ? selection_color() : color();
-//if (col == FL_GRAY && Fl::belowmouse()==this) col = FL_LIGHT1;
   draw_box(value() ? (down_box()?down_box():fl_down(box())) : box(), col);
-  draw_label();
+  draw_backdrop();
+  if (labeltype() == FL_NORMAL_LABEL && value()) {
+    Fl_Color c = labelcolor();
+    labelcolor(fl_contrast(c, col));
+    draw_label();
+    labelcolor(c);
+  } else draw_label();
   if (Fl::focus() == this) draw_focus();
 }
 
 int Fl_Button::handle(int event) {
   int newval;
   switch (event) {
-  case FL_ENTER:
+  case FL_ENTER: /* FALLTHROUGH */
   case FL_LEAVE:
 //  if ((value_?selection_color():color())==FL_GRAY) redraw();
     return 1;
@@ -78,9 +100,13 @@ int Fl_Button::handle(int event) {
       if (type() == FL_RADIO_BUTTON) newval = 1;
       else newval = !oldval;
     } else
+    {
+      clear_changed();
       newval = oldval;
+    }
     if (newval != value_) {
       value_ = newval;
+      set_changed();
       redraw();
       if (when() & FL_WHEN_CHANGED) do_callback();
     }
@@ -90,32 +116,26 @@ int Fl_Button::handle(int event) {
       if (when() & FL_WHEN_NOT_CHANGED) do_callback();
       return 1;
     }
-    if (type() == FL_RADIO_BUTTON)
-      setonly();
-    else if (type() == FL_TOGGLE_BUTTON)
-      oldval = value_;
+    set_changed();
+    if (type() == FL_RADIO_BUTTON) setonly();
+    else if (type() == FL_TOGGLE_BUTTON) oldval = value_;
     else {
       value(oldval);
-      if (when() & FL_WHEN_CHANGED) do_callback();
+      set_changed();
+      if (when() & FL_WHEN_CHANGED) {
+	Fl_Widget_Tracker wp(this);
+        do_callback();
+        if (wp.deleted()) return 1;
+      }
     }
-    if (when() & FL_WHEN_RELEASE) do_callback(); else set_changed();
+    if (when() & FL_WHEN_RELEASE) do_callback();
     return 1;
   case FL_SHORTCUT:
     if (!(shortcut() ?
-	  Fl::test_shortcut(shortcut()) : test_shortcut())) return 0;
-    
+	  Fl::test_shortcut(shortcut()) : test_shortcut())) return 0;    
     if (Fl::visible_focus() && handle(FL_FOCUS)) Fl::focus(this);
-
-    if (type() == FL_RADIO_BUTTON && !value_) {
-      setonly();
-      if (when() & FL_WHEN_CHANGED) do_callback();
-    } else if (type() == FL_TOGGLE_BUTTON) {
-      value(!value());
-      if (when() & FL_WHEN_CHANGED) do_callback();
-    }
-    if (when() & FL_WHEN_RELEASE) do_callback(); else set_changed();
-    return 1;
-  case FL_FOCUS :
+    goto triggered_by_keyboard;
+  case FL_FOCUS : /* FALLTHROUGH */
   case FL_UNFOCUS :
     if (Fl::visible_focus()) {
       if (box() == FL_NO_BOX) {
@@ -131,14 +151,20 @@ int Fl_Button::handle(int event) {
   case FL_KEYBOARD :
     if (Fl::focus() == this && Fl::event_key() == ' ' &&
         !(Fl::event_state() & (FL_SHIFT | FL_CTRL | FL_ALT | FL_META))) {
+      set_changed();
+    triggered_by_keyboard:
+      Fl_Widget_Tracker wp(this);
       if (type() == FL_RADIO_BUTTON && !value_) {
 	setonly();
 	if (when() & FL_WHEN_CHANGED) do_callback();
       } else if (type() == FL_TOGGLE_BUTTON) {
 	value(!value());
 	if (when() & FL_WHEN_CHANGED) do_callback();
+      } else {
+        simulate_key_action();
       }
-      if (when() & FL_WHEN_RELEASE) do_callback(); else set_changed();
+      if (wp.deleted()) return 1;
+      if (when() & FL_WHEN_RELEASE) do_callback();
       return 1;
     }
   default:
@@ -146,8 +172,40 @@ int Fl_Button::handle(int event) {
   }
 }
 
-Fl_Button::Fl_Button(int X, int Y, int W, int H, const char *l)
-: Fl_Widget(X,Y,W,H,l) {
+void Fl_Button::simulate_key_action()
+{
+  if (key_release_tracker) {
+    Fl::remove_timeout(key_release_timeout, key_release_tracker);
+    key_release_timeout(key_release_tracker);
+  }
+  value(1); 
+  redraw();
+  key_release_tracker = new Fl_Widget_Tracker(this);
+  Fl::add_timeout(0.15, key_release_timeout, key_release_tracker);
+}
+
+void Fl_Button::key_release_timeout(void *d)
+{
+  Fl_Widget_Tracker *wt = (Fl_Widget_Tracker*)d;
+  if (!wt)
+    return;
+  if (wt==key_release_tracker) 
+    key_release_tracker = 0L;
+  Fl_Button *btn = (Fl_Button*)wt->widget();
+  if (btn) {
+    btn->value(0);
+    btn->redraw();
+  }
+  delete wt;
+}
+
+/**
+  The constructor creates the button using the given position, size and label.
+  \param[in] X, Y, W, H position and size of the widget
+  \param[in] L widget label, default is no label
+ */
+Fl_Button::Fl_Button(int X, int Y, int W, int H, const char *L)
+: Fl_Widget(X,Y,W,H,L) {
   box(FL_UP_BOX);
   down_box(FL_NO_BOX);
   value_ = oldval = 0;
@@ -156,5 +214,5 @@ Fl_Button::Fl_Button(int X, int Y, int W, int H, const char *l)
 }
 
 //
-// End of "$Id: Fl_Button.cxx,v 1.4.2.6.2.21 2003/01/30 21:41:30 easysw Exp $".
+// End of "$Id: Fl_Button.cxx 7903 2010-11-28 21:06:39Z matt $".
 //
