@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Clock.cxx,v 1.8.2.3 2000/06/05 21:20:49 mike Exp $"
+// "$Id: Fl_Clock.cxx 7903 2010-11-28 21:06:39Z matt $"
 //
 // Clock widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2000 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 #include <FL/Fl.H>
@@ -52,6 +54,10 @@ static void drawhand(double ang,const float v[][2],Fl_Color fill,Fl_Color line)
 }
 
 void Fl_Clock_Output::drawhands(Fl_Color fill, Fl_Color line) {
+  if (!active_r()) {
+    fill = fl_inactive(fill);
+    line = fl_inactive(line);
+  }
   drawhand(-360*(hour()+minute()/60.0)/12, hourhand, fill, line);
   drawhand(-360*(minute()+second()/60.0)/60, minhand, fill, line);
   drawhand(-360*(second()/60.0), sechand, fill, line);
@@ -68,25 +74,31 @@ static void rect(double x, double y, double w, double h) {
   fl_end_polygon();
 }
 
-void Fl_Clock_Output::draw(int x, int y, int w, int h) {
-  draw_box(box(), x, y, w, h, type()==FL_ROUND_CLOCK ? FL_GRAY : color());
+/**
+  Draw clock with the given position and size.
+  \param[in] X, Y, W, H position and size
+*/
+void Fl_Clock_Output::draw(int X, int Y, int W, int H) {
+  Fl_Color box_color = type()==FL_ROUND_CLOCK ? FL_GRAY : color();
+  Fl_Color shadow_color = fl_color_average(box_color, FL_BLACK, 0.5);
+  draw_box(box(), X, Y, W, H, box_color);
   fl_push_matrix();
-  fl_translate(x+w/2.0-.5, y+h/2.0-.5);
-  fl_scale((w-1)/28.0, (h-1)/28.0);
+  fl_translate(X+W/2.0-.5, Y+H/2.0-.5);
+  fl_scale((W-1)/28.0, (H-1)/28.0);
   if (type() == FL_ROUND_CLOCK) {
-    fl_color(color());
+    fl_color(active_r() ? color() : fl_inactive(color()));
     fl_begin_polygon(); fl_circle(0,0,14); fl_end_polygon();
-    fl_color(FL_BLACK);
+    fl_color(active_r() ? FL_FOREGROUND_COLOR : fl_inactive(FL_FOREGROUND_COLOR));
     fl_begin_loop(); fl_circle(0,0,14); fl_end_loop();
   }
   // draw the shadows:
   fl_push_matrix();
   fl_translate(0.60, 0.60);
-  drawhands(FL_DARK3, FL_DARK3);
+  drawhands(shadow_color, shadow_color);
   fl_pop_matrix();
   // draw the tick marks:
   fl_push_matrix();
-  fl_color(FL_BLACK); // color was 52
+  fl_color(active_r() ? FL_FOREGROUND_COLOR : fl_inactive(FL_FOREGROUND_COLOR));
   for (int i=0; i<12; i++) {
     if (i==6) rect(-0.5, 9, 1, 2);
     else if (i==3 || i==0 || i== 9) rect(-0.5, 9.5, 1, 1);
@@ -95,30 +107,55 @@ void Fl_Clock_Output::draw(int x, int y, int w, int h) {
   }
   fl_pop_matrix();
   // draw the hands:
-  drawhands(selection_color(), FL_GRAY0); // color was 54
+  drawhands(selection_color(), FL_FOREGROUND_COLOR); // color was 54
   fl_pop_matrix();
 }
 
+/**
+  Draw clock with current position and size.
+*/
 void Fl_Clock_Output::draw() {
   draw(x(), y(), w(), h());
   draw_label();
 }
 
-void Fl_Clock_Output::value(int h, int m, int s) {
-  if (h!=hour_ || m!=minute_ || s!=second_) {
-    hour_ = h; minute_ = m; second_ = s;
+/**
+  Set the displayed time.
+  Set the time in hours, minutes, and seconds.
+  \param[in] H, m, s displayed time
+  \see hour(), minute(), second()
+ */
+void Fl_Clock_Output::value(int H, int m, int s) {
+  if (H!=hour_ || m!=minute_ || s!=second_) {
+    hour_ = H; minute_ = m; second_ = s;
+    value_ = (H * 60 + m) * 60 + s;
     damage(FL_DAMAGE_CHILD);
   }
 }
 
+/**
+  Set the displayed time.
+  Set the time in seconds since the UNIX epoch (January 1, 1970).
+  \param[in] v seconds since epoch
+  \see value()
+ */
 void Fl_Clock_Output::value(ulong v) {
+  value_ = v;
   struct tm *timeofday;
-  timeofday = localtime((const time_t *)&v);
+  // Some platforms, notably Windows, now use a 64-bit time_t value...
+  time_t vv = (time_t)v;
+  timeofday = localtime(&vv);
   value(timeofday->tm_hour, timeofday->tm_min, timeofday->tm_sec);
 }
 
-Fl_Clock_Output::Fl_Clock_Output(int x, int y, int w, int h, const char *l)
-: Fl_Widget(x, y, w, h, l) {
+/**
+  Create a new Fl_Clock_Output widget with the given position, size and label.
+  The default boxtype is \c FL_NO_BOX.
+  \param[in] X, Y, W, H position and size of the widget
+  \param[in] L widget label, default is no label
+ */
+Fl_Clock_Output::Fl_Clock_Output(int X, int Y, int W, int H, const char *L)
+: Fl_Widget(X, Y, W, H, L) {
   box(FL_UP_BOX);
   selection_color(fl_gray_ramp(5));
   align(FL_ALIGN_BOTTOM);
@@ -130,27 +167,31 @@ Fl_Clock_Output::Fl_Clock_Output(int x, int y, int w, int h, const char *l)
 
 ////////////////////////////////////////////////////////////////
 
-Fl_Clock::Fl_Clock(int x, int y, int w, int h, const char *l)
-  : Fl_Clock_Output(x, y, w, h, l) {}
+/**
+  Create an Fl_Clock widget using the given position, size, and label string.
+  The default boxtype is \c FL_NO_BOX.
+  \param[in] X, Y, W, H position and size of the widget
+  \param[in] L widget label, default is no label
+ */
+Fl_Clock::Fl_Clock(int X, int Y, int W, int H, const char *L)
+  : Fl_Clock_Output(X, Y, W, H, L) {}
 
-Fl_Clock::Fl_Clock(uchar t, int x, int y, int w, int h, const char *l)
-  : Fl_Clock_Output(x, y, w, h, l) {
+/**
+  Create an Fl_Clock widget using the given boxtype, position, size, and
+  label string.
+  \param[in] t boxtype
+  \param[in] X, Y, W, H position and size of the widget
+  \param[in] L widget label, default is no label
+ */
+Fl_Clock::Fl_Clock(uchar t, int X, int Y, int W, int H, const char *L)
+  : Fl_Clock_Output(X, Y, W, H, L) {
   type(t);
   box(t==FL_ROUND_CLOCK ? FL_NO_BOX : FL_UP_BOX);
 }
 
 static void tick(void *v) {
-#ifdef WIN32
   ((Fl_Clock*)v)->value(time(0));
   Fl::add_timeout(1.0, tick, v);
-#else
-  struct timeval t;
-  gettimeofday(&t, 0);
-  ((Fl_Clock*)v)->value(t.tv_sec);
-  double delay = 1.0-t.tv_usec*.000001;
-  if (delay < .1 || delay > .9) delay = 1.0;
-  Fl::add_timeout(delay, tick, v);
-#endif
 }
 
 int Fl_Clock::handle(int event) {
@@ -165,10 +206,13 @@ int Fl_Clock::handle(int event) {
   return Fl_Clock_Output::handle(event);
 }
   
+/**
+  The destructor removes the clock.
+ */
 Fl_Clock::~Fl_Clock() {
   Fl::remove_timeout(tick, this);
 }
 
 //
-// End of "$Id: Fl_Clock.cxx,v 1.8.2.3 2000/06/05 21:20:49 mike Exp $".
+// End of "$Id: Fl_Clock.cxx 7903 2010-11-28 21:06:39Z matt $".
 //
