@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Scrollbar.cxx,v 1.7.2.11 2000/06/05 21:20:57 mike Exp $"
+// "$Id: Fl_Scrollbar.cxx 7903 2010-11-28 21:06:39Z matt $"
 //
 // Scroll bar widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2000 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,33 +20,48 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
 //
+//     http://www.fltk.org/str.php
+//
+
 
 #include <FL/Fl.H>
 #include <FL/Fl_Scrollbar.H>
 #include <FL/fl_draw.H>
 #include <math.h>
+#include "flstring.h"
 
 #define INITIALREPEAT .5
 #define REPEAT .05
 
 void Fl_Scrollbar::increment_cb() {
-  int ls = maximum()>=minimum() ? linesize_ : -linesize_;
+  char inv = maximum()<minimum();
+  int ls = inv ? -linesize_ : linesize_;
   int i;
   switch (pushed_) {
-  case 1:
-    i = -ls;
-    break;
-  default:
-    i =  ls;
-    break;
-  case 5:
-    i = -int((maximum()-minimum())*slider_size()/(1.0-slider_size())) + ls;
-    break;
-  case 6:
-    i =  int((maximum()-minimum())*slider_size()/(1.0-slider_size())) - ls;
-    break;
+    case 1: // clicked on arrow left
+      i = -ls;
+      break;
+    default: // clicked on arrow right
+      i =  ls;
+      break;
+    case 5: // clicked into the box next to the slider on the left
+      i = -(int((maximum()-minimum())*slider_size()/(1.0-slider_size())));
+      if (inv) {
+        if (i<-ls) i = -ls;
+      } else {
+        if (i>-ls) i = -ls; // err
+      }
+      break;
+    case 6: // clicked into the box next to the slider on the right
+      i = (int((maximum()-minimum())*slider_size()/(1.0-slider_size())));
+      if (inv) {
+        if (i>ls) i = ls;
+      } else {
+        if (i<ls) i = ls; // err
+      }
+      break;
   }
   handle_drag(clamp(value() + i));
 }
@@ -84,6 +99,7 @@ int Fl_Scrollbar::handle(int event) {
   else {
     int S = int(slider_size()*ww+.5);
     int T = (horizontal() ? H : W)/2+1;
+    if (type()==FL_VERT_NICE_SLIDER || type()==FL_HOR_NICE_SLIDER) T += 4;
     if (S < T) S = T;
     double val =
       (maximum()-minimum()) ? (value()-minimum())/(maximum()-minimum()) : 0.5;
@@ -91,7 +107,8 @@ int Fl_Scrollbar::handle(int event) {
     if (val >= 1.0) sliderx = ww-S;
     else if (val <= 0.0) sliderx = 0;
     else sliderx = int(val*(ww-S)+.5);
-    if (relx < sliderx) area = 5;
+    if (Fl::event_button() == FL_MIDDLE_MOUSE) area = 8;
+    else if (relx < sliderx) area = 5;
     else if (relx >= sliderx+S) area = 6;
     else area = 8;
   }
@@ -122,7 +139,20 @@ int Fl_Scrollbar::handle(int event) {
   case FL_DRAG:
     if (pushed_) return 1;
     return Fl_Slider::handle(event, X,Y,W,H);
-  case FL_SHORTCUT: {
+  case FL_MOUSEWHEEL :
+    if (horizontal()) {
+      if (Fl::e_dx==0) return 0;
+      int ls = maximum()>=minimum() ? linesize_ : -linesize_;
+      handle_drag(clamp(value() + ls * Fl::e_dx));
+      return 1;
+    } else {
+      if (Fl::e_dy==0) return 0;
+      int ls = maximum()>=minimum() ? linesize_ : -linesize_;
+      handle_drag(clamp(value() + ls * Fl::e_dy));
+      return 1;
+    }
+  case FL_SHORTCUT:
+  case FL_KEYBOARD: {
     int v = value();
     int ls = maximum()>=minimum() ? linesize_ : -linesize_;
     if (horizontal()) {
@@ -168,6 +198,7 @@ int Fl_Scrollbar::handle(int event) {
     if (v != value()) {
       Fl_Slider::value(v);
       value_damage();
+      set_changed();
       do_callback();
     }
     return 1;}
@@ -185,46 +216,62 @@ void Fl_Scrollbar::draw() {
     if (W < 3*H) {Fl_Slider::draw(X,Y,W,H); return;}
     Fl_Slider::draw(X+H,Y,W-2*H,H);
     if (damage()&FL_DAMAGE_ALL) {
-      draw_box((pushed_==1) ? down(slider()) : slider(),
+      draw_box((pushed_==1) ? fl_down(slider()) : slider(),
 	       X, Y, H, H, selection_color());
-      draw_box((pushed_==2) ? down(slider()) : slider(),
-		  X+W-H, Y, H, H, selection_color());
+      draw_box((pushed_==2) ? fl_down(slider()) : slider(),
+	       X+W-H, Y, H, H, selection_color());
       if (active_r())
         fl_color(labelcolor());
       else
-        fl_color(inactive(labelcolor()));
+        fl_color(fl_inactive(labelcolor()));
       int w1 = (H-4)/3; if (w1 < 1) w1 = 1;
       int x1 = X+(H-w1-1)/2;
-      int y1 = Y+(H-2*w1-1)/2;
-      fl_polygon(x1, y1+w1, x1+w1, y1+2*w1, x1+w1, y1);
-      x1 += (W-H);
-      fl_polygon(x1, y1, x1, y1+2*w1, x1+w1, y1+w1);
+      int yy1 = Y+(H-2*w1-1)/2;
+      if (Fl::scheme_ && !strcmp(Fl::scheme_, "gtk+")) {
+	fl_polygon(x1, yy1+w1, x1+w1, yy1+2*w1, x1+w1-1, yy1+w1, x1+w1, yy1);
+	x1 += (W-H);
+	fl_polygon(x1, yy1, x1+1, yy1+w1, x1, yy1+2*w1, x1+w1, yy1+w1);
+      } else {
+	fl_polygon(x1, yy1+w1, x1+w1, yy1+2*w1, x1+w1, yy1);
+	x1 += (W-H);
+	fl_polygon(x1, yy1, x1, yy1+2*w1, x1+w1, yy1+w1);
+      }
     }
   } else { // vertical
     if (H < 3*W) {Fl_Slider::draw(X,Y,W,H); return;}
     Fl_Slider::draw(X,Y+W,W,H-2*W);
     if (damage()&FL_DAMAGE_ALL) {
-      draw_box((pushed_==1) ? down(slider()) : slider(),
+      draw_box((pushed_==1) ? fl_down(slider()) : slider(),
 	       X, Y, W, W, selection_color());
-      draw_box((pushed_==2) ? down(slider()) : slider(),
+      draw_box((pushed_==2) ? fl_down(slider()) : slider(),
 	       X, Y+H-W, W, W, selection_color());
       if (active_r())
         fl_color(labelcolor());
       else
-        fl_color(labelcolor() | 8);
+        fl_color(fl_inactive(labelcolor()));
       int w1 = (W-4)/3; if (w1 < 1) w1 = 1;
       int x1 = X+(W-2*w1-1)/2;
-      int y1 = Y+(W-w1-1)/2;
-      fl_polygon(x1, y1+w1, x1+2*w1, y1+w1, x1+w1, y1);
-      y1 += H-W;
-      fl_polygon(x1, y1, x1+w1, y1+w1, x1+2*w1, y1);
+      int yy1 = Y+(W-w1-1)/2;
+      if (Fl::scheme_ && !strcmp(Fl::scheme_, "gtk+")) {
+	fl_polygon(x1, yy1+w1, x1+w1, yy1+w1-1, x1+2*w1, yy1+w1, x1+w1, yy1);
+	yy1 += H-W;
+	fl_polygon(x1, yy1, x1+w1, yy1+1, x1+w1, yy1+w1);
+	fl_polygon(x1+w1, yy1+1, x1+2*w1, yy1, x1+w1, yy1+w1);
+      } else {
+	fl_polygon(x1, yy1+w1, x1+2*w1, yy1+w1, x1+w1, yy1);
+	yy1 += H-W;
+	fl_polygon(x1, yy1, x1+w1, yy1+w1, x1+2*w1, yy1);
+      }
     }
   }
 }
 
+/**
+  Creates a new Fl_Scrollbar widget with given position, size, and label.
+  You need to do type(FL_HORIZONTAL) if you want a horizontal scrollbar.
+*/
 Fl_Scrollbar::Fl_Scrollbar(int X, int Y, int W, int H, const char* L)
-  : Fl_Slider(X, Y, W, H, L)
-{
+  : Fl_Slider(X, Y, W, H, L) {
   box(FL_FLAT_BOX);
   color(FL_DARK2);
   slider(FL_UP_BOX);
@@ -233,6 +280,13 @@ Fl_Scrollbar::Fl_Scrollbar(int X, int Y, int W, int H, const char* L)
   step(1);
 }
 
+/**  Destroys the Scrollbar. */
+Fl_Scrollbar::~Fl_Scrollbar() {
+  if (pushed_)
+    Fl::remove_timeout(timeout_cb, this);
+}
+
+
 //
-// End of "$Id: Fl_Scrollbar.cxx,v 1.7.2.11 2000/06/05 21:20:57 mike Exp $".
+// End of "$Id: Fl_Scrollbar.cxx 7903 2010-11-28 21:06:39Z matt $".
 //
