@@ -1,9 +1,9 @@
 //
-// "$Id: fl_color_mac.cxx 4288 2005-04-16 00:13:17Z mike $"
+// "$Id: fl_color_mac.cxx 8384 2011-02-06 12:32:23Z manolo $"
 //
 // MacOS color functions for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -47,10 +47,8 @@ Fl_XMap fl_xmap[256];
 
 Fl_XMap* fl_current_xmap;
 
-Fl_Color fl_color_;
-
-void fl_color(Fl_Color i) {
-  fl_color_ = i;
+void Fl_Quartz_Graphics_Driver::color(Fl_Color i) {
+  Fl_Graphics_Driver::color(i);
   int index;
   uchar r, g, b;
   if (i & 0xFFFFFF00) {
@@ -66,41 +64,21 @@ void fl_color(Fl_Color i) {
     g = c>>16;
     b = c>> 8;
   }
-#ifdef __APPLE_QD__
-  RGBColor rgb; 
-  rgb.red   = (r<<8)|r;
-  rgb.green = (g<<8)|g;
-  rgb.blue  = (b<<8)|b;
-  RGBForeColor(&rgb);
-#elif defined(__APPLE_QUARTZ__)
   if (!fl_gc) return; // no context yet? We will assign the color later.
   float fr = r/255.0f;
   float fg = g/255.0f;
   float fb = b/255.0f;
   CGContextSetRGBFillColor(fl_gc, fr, fg, fb, 1.0f);
   CGContextSetRGBStrokeColor(fl_gc, fr, fg, fb, 1.0f);
-#else
-#  error : neither Quickdraw nor Quartz defined
-#endif
 }
 
-void fl_color(uchar r, uchar g, uchar b) {
-  fl_color_ = fl_rgb_color(r, g, b);
-#ifdef __APPLE_QD__
-  RGBColor rgb; 
-  rgb.red   = (r<<8)|r;
-  rgb.green = (g<<8)|g;
-  rgb.blue  = (b<<8)|b;
-  RGBForeColor(&rgb);
-#elif defined(__APPLE_QUARTZ__)
+void Fl_Quartz_Graphics_Driver::color(uchar r, uchar g, uchar b) {
+  Fl_Graphics_Driver::color( fl_rgb_color(r, g, b) );
   float fr = r/255.0f;
   float fg = g/255.0f;
   float fb = b/255.0f;
   CGContextSetRGBFillColor(fl_gc, fr, fg, fb, 1.0f);
   CGContextSetRGBStrokeColor(fl_gc, fr, fg, fb, 1.0f);
-#else
-#  error : neither Quickdraw nor Quartz defined
-#endif
 }
 
 void Fl::set_color(Fl_Color i, unsigned c) {
@@ -110,5 +88,5 @@ void Fl::set_color(Fl_Color i, unsigned c) {
 }
 
 //
-// End of "$Id: fl_color_mac.cxx 4288 2005-04-16 00:13:17Z mike $".
+// End of "$Id: fl_color_mac.cxx 8384 2011-02-06 12:32:23Z manolo $".
 //
