@@ -1,9 +1,9 @@
 //
-// "$Id: fl_file_dir.cxx 4288 2005-04-16 00:13:17Z mike $"
+// "$Id: fl_file_dir.cxx 8345 2011-01-31 18:04:09Z manolo $"
 //
 // File chooser widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -42,30 +42,48 @@ static void callback(Fl_File_Chooser *, void*) {
     (*current_callback)(fc->value());
 }
 
+/** \addtogroup  group_comdlg 
+    @{ */
 
-// Set the file chooser callback
+/** 
+    Set the file chooser callback
+    \note \#include <FL/Fl_File_Chooser.H>
+    \relates Fl_File_Chooser
+*/
 void fl_file_chooser_callback(void (*cb)(const char*)) {
   current_callback = cb;
 }
 
 
-// Set the "OK" button label
+/** 
+    Set the "OK" button label
+    \note \#include <FL/Fl_File_Chooser.H>
+    \relates Fl_File_Chooser
+*/
 void fl_file_chooser_ok_label(const char *l) {
   if (l) current_label = l;
   else current_label = fl_ok;
 }
 
-
-//
-// 'fl_file_chooser()' - Show a file chooser dialog and get a filename.
-//
-
+/** 
+    Shows a file chooser dialog and gets a filename. 
+    \note \#include <FL/Fl_File_Chooser.H>
+    \image html Fl_File_Chooser.jpg 
+    \image latex  Fl_File_Chooser.jpg "Fl_File_Chooser" width=12cm
+    \param[in] message text in title bar
+    \param[in] pat filename pattern filter
+    \param[in] fname initial/default filename selection
+    \param[in] relative 0 for absolute path name, relative path name otherwise
+    \return the user selected filename, in absolute or relative format 
+            or NULL if user cancels
+    \relates Fl_File_Chooser
+*/
 char *					// O - Filename or NULL
 fl_file_chooser(const char *message,	// I - Message in titlebar
                 const char *pat,	// I - Filename pattern
 		const char *fname,	// I - Initial filename selection
 		int        relative) {	// I - 0 for absolute path
-  static char	retname[1024];		// Returned filename
+  static char	retname[FL_PATH_MAX];		// Returned filename
 
   if (!fc) {
     if (!fname || !*fname) fname = ".";
@@ -74,12 +92,19 @@ fl_file_chooser(const char *message,	// I - Message in titlebar
     fc->callback(callback, 0);
   } else {
     fc->type(Fl_File_Chooser::CREATE);
+    // see, if we use the same pattern between calls
+    char same_pattern = 0;
+    const char *fcf = fc->filter();
+    if ( fcf && pat && strcmp(fcf, pat)==0)
+      same_pattern = 1;
+    else if ( (fcf==0L || *fcf==0) && (pat==0L || *pat==0) )
+      same_pattern = 1;
+    // now set the pattern to the new pattern (even if they are the same)
     fc->filter(pat);
     fc->label(message);
 
-    if (!fname || !*fname) {
-      if (fc->filter() != pat && (!pat || !fc->filter() ||
-          strcmp(pat, fc->filter())) && fc->value()) {
+    if (!fname) { // null pointer reuses same filename if pattern didn't change
+      if (!same_pattern && fc->value()) {
 	// if pattern is different, remove name but leave old directory:
 	strlcpy(retname, fc->value(), sizeof(retname));
 
@@ -93,13 +118,24 @@ fl_file_chooser(const char *message,	// I - Message in titlebar
 	  else
 	    *p = '\0';
 	}
-
 	// Set the directory...
-	fc->directory(retname);
+	fc->value(retname);
+      } else {
+        // re-use the previously selected name
       }
+    } else if (!*fname) { // empty filename reuses directory with empty name
+      const char *fcv = fc->value();
+      if (fcv) 
+        strlcpy(retname, fc->value(), sizeof(retname));
+      else 
+        *retname = 0;
+      const char *n = fl_filename_name(retname);
+      if (n) *((char*)n) = 0;
+      fc->value("");
+      fc->directory(retname);
+    } else {
+       fc->value(fname);
     }
-    else
-      fc->value(fname);
   }
 
   fc->ok_label(current_label);
@@ -116,17 +152,20 @@ fl_file_chooser(const char *message,	// I - Message in titlebar
   else return 0;
 }
 
-
-//
-// 'fl_dir_chooser()' - Show a file chooser dialog and get a directory.
-//
-
+/** Shows a file chooser dialog and gets a directory.
+    \note \#include <FL/Fl_File_Chooser.H>
+    \param[in] message title bar text
+    \param[in] fname initial/default directory name
+    \param[in] relative 0 for absolute path return, relative otherwise
+    \return the directory path string chosen by the user or NULL if user cancels
+    \relates Fl_File_Chooser
+*/
 char *					// O - Directory or NULL
 fl_dir_chooser(const char *message,	// I - Message for titlebar
                const char *fname,	// I - Initial directory name
 	       int        relative)	// I - 0 for absolute
 {
-  static char	retname[1024];		// Returned directory name
+  static char	retname[FL_PATH_MAX];		// Returned directory name
 
   if (!fc) {
     if (!fname || !*fname) fname = ".";
@@ -153,8 +192,9 @@ fl_dir_chooser(const char *message,	// I - Message for titlebar
   } else if (fc->value()) return (char *)fc->value();
   else return 0;
 }
+/** @} */
 
 
 //
-// End of "$Id: fl_file_dir.cxx 4288 2005-04-16 00:13:17Z mike $".
+// End of "$Id: fl_file_dir.cxx 8345 2011-01-31 18:04:09Z manolo $".
 //
