@@ -1,9 +1,9 @@
 //
-// "$Id: fl_images_core.cxx,v 1.1.2.5 2003/01/30 21:43:55 easysw Exp $"
+// "$Id: fl_images_core.cxx 7903 2010-11-28 21:06:39Z matt $"
 //
 // FLTK images library core.
 //
-// Copyright 1997-2003 by Easy Software Products.
+// Copyright 1997-2010 by Easy Software Products.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 // Contents:
 //
@@ -51,10 +53,13 @@
 static Fl_Image	*fl_check_images(const char *name, uchar *header, int headerlen);
 
 
-//
-// 'fl_register_images()' - Register the image formats.
-//
-
+/**
+\brief Register the image formats.
+ *
+ This function is provided in the fltk_images library and 
+ registers all of the "extra" image file formats that are not part
+ of the core FLTK library.
+*/
 void fl_register_images() {
   Fl_Shared_Image::add_handler(fl_check_images);
 }
@@ -67,7 +72,7 @@ void fl_register_images() {
 Fl_Image *					// O - Image, if found
 fl_check_images(const char *name,		// I - Filename
                 uchar      *header,		// I - Header data from file
-		int        headerlen) {		// I - Amount of data
+		int) {				// I - Amount of data (not used)
   if (memcmp(header, "GIF87a", 6) == 0 ||
       memcmp(header, "GIF89a", 6) == 0)	// GIF file
     return new Fl_GIF_Image(name);
@@ -87,7 +92,7 @@ fl_check_images(const char *name,		// I - Filename
 #ifdef HAVE_LIBJPEG
   if (memcmp(header, "\377\330\377", 3) == 0 &&
 					// Start-of-Image
-      header[3] >= 0xe0 && header[3] <= 0xef)
+      header[3] >= 0xc0 && header[3] <= 0xef)
 	   				// APPn for JPEG file
     return new Fl_JPEG_Image(name);
 #endif // HAVE_LIBJPEG
@@ -97,5 +102,5 @@ fl_check_images(const char *name,		// I - Filename
 
 
 //
-// End of "$Id: fl_images_core.cxx,v 1.1.2.5 2003/01/30 21:43:55 easysw Exp $".
+// End of "$Id: fl_images_core.cxx 7903 2010-11-28 21:06:39Z matt $".
 //
